@@ -1,39 +1,29 @@
 package Mongoose::Cursor;
-$Mongoose::Cursor::VERSION = '0.31';
+$Mongoose::Cursor::VERSION = '0.32';
 use Moose;
-use MongoDB;
 extends 'MongoDB::Cursor';
 
-has '_class' => ( is=>'rw', isa=>'Str', required=>1 );
-has '_collection_name' => ( is=>'rw', isa=>'Str', required=>1 );
+has _class           => ( is=>'rw', isa=>'Str', required=>1 );
+has _collection_name => ( is=>'rw', isa=>'Str', required=>1 );
 
 around 'next' => sub {
     my ($orig,$self, @args)=@_;
-    my $doc = $self->$orig(@args);
-    return unless defined $doc;
-    my $coll_name = $self->_collection_name; 
+    my $doc   = $self->$orig(@args) || return;
     my $class = $self->_class;
-    #eval "require " . $self->_class;
-    return $class->expand( $doc );
+    $class->expand( $doc );
 };
 
 around 'all' => sub {
     my ($orig,$self, @args)=@_;
-    my @docs = $self->$orig(@args);
-
-    return unless scalar @docs > 0;
-
-    my $coll_name = $self->_collection_name; 
     my $class = $self->_class;
-
-    return map { $class->expand( $_ ) } @docs;
+    map { $class->expand( $_ ) } $self->$orig(@args);
 };
 
 sub each(&) {
     my $self = shift;
     my $func = shift;
     while( my $r = $self->next ) {
-        last unless defined $func->( $r ) 
+        last unless defined $func->( $r )
     }
 }
 
@@ -63,7 +53,7 @@ Mongoose::Cursor - a Mongoose wrapper for MongoDB::Cursor
 
 =head1 DESCRIPTION
 
-Extends L<Mongoose::Cursor>. 
+Extends L<Mongoose::Cursor>.
 
 Wraps L<MongoDB::Cursor>'s C<next> and C<all>methods,
 so that it expands a document into a class.
